--MapController
MapController = Controller('MapController')

-----------------------
-- Variables:
-----------------------

TILE_PASS = 1
TILE_BLOCK = 0

function MapController:onInit()
	EventDispatcher.listen("on_TileMovementController_interval", nil, self.onTileMovementControllerInterval, self )
end


-----------------------
-- Dependencies:
-----------------------

function MapController:onControllersRefresh( cm )
	self.map = entityManager:getComponentFromEntity( currentMap, MapAttribute )
end

-----------------------
-- Events:
-----------------------

function MapController:onTileMovementControllerInterval( tileMovementAspect )
	if entityManager:getEntityName( tileMovementAspect ) == 'PlayerCharacter' then
		local mapPosition = tileMovementAspect.MapPositionAttribute.position
		local tileIndex = self.mapdata.width * mapPosition.y + mapPosition.x + 1
		local tileID = self.map.tilemap[tileIndex]
		if tileID == TILE_PASS then
			-- to change a tile's ID programmatically: self.map.tilemap[tileIndex] = TILE_PASS
			eventDispatcher.send("on_tileMovementAspect_pass", nil, tileMovementAspect) -- notify callback
		elseif tileID == TILE_BLOCK then
			eventDispatcher.send("on_tileMovementAspect_block", nil, tileMovementAspect)
		end
	end
end

-----------------------
-- Helpers:
-----------------------

function MapController:getTileIDAtCoords( x, y )
	local index = self.map.leveldata.width * y + x + 1
	return self.map.tilemap[index]
end

--
function MapController:isTileIDAFence( tileID )
	for _, fenceID in ipairs(TILE_FENCE) do
		if tileID == fenceID then
			return true
		end
	end
	return false
end

-----------------------
-- Update:
-----------------------
--TODO: add camera functionality
function MapController:onUpdate()
	local mapdata	= self.map.leveldata
	local tileset	= self.map.tileset
	tileset.spriteBatch:clear()
	local x = 0
	local y = 0
	local tilewidth  = mapdata.tilewidth
	local tileheight = mapdata.tileheight
	local mapwidth   = mapdata.width
	local mapheight  = mapdata.height
	for i, tileID in ipairs( self.map.tilemap ) do
		tileset.spriteBatch:addq( tileset.quads[tileID], x * tilewidth, y * tileheight )
		x = x + 1
		if x >= mapwidth then
			x = 0
			y = y + 1
		end
	end
end
